#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# Trashman v1.0.2
# A Python trash manager.
# Copyright (C) 2011-2012, Kwpolska.
# See /LICENSE for licensing information.

"""
    trashman.backends.trash
    ~~~~~~~~~~~~~~~~~~~~~~~

    Base Trash class.

    :Copyright: © 2011-2012, Kwpolska.
    :License: BSD (see /LICENSE).
"""
from .. import DS


### Trash          Base Trash class        ###
class Trash(object):
    """Base Trash class."""

    trashdir = None
    filedir = None
    infodir = None
    logger = DS.log

    def log(self, msg, lvl='debug'):
        """Log a message."""
        if lvl in ('debug', 'info', 'warning', 'error', 'critical'):
            eval('self.logger.{}(msg)'.format(lvl))
        else:
            DS.critical('SECURITY ALERT: Trash.log got a wrong level!')

    def regenerate(self):
        """Regenerate the trash and recreate metadata."""
        pass  # Some backends don’t need regeneration.

    def empty(self, verbose):
        """Empty the trash."""
        raise NotImplementedError(_('Backend didn’t \
implement this functionality'))

    def list(self, human=True):
        """List the trash contents."""
        raise NotImplementedError(_('Backend didn’t \
implement this functionality'))

    def trash(self, filepath, verbose):
        """Move specified file to trash."""
        raise NotImplementedError(_('Backend didn’t \
implement this functionality'))

    def restore(self, filename, verbose):
        """Restore a file from trash."""
        raise NotImplementedError(_('Backend didn’t \
implement this functionality'))



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# Trashman v1.0.2
# A Python trash manager.
# Copyright (C) 2011-2012, Kwpolska.
# See /LICENSE for licensing information.

"""
    trashman.backends
    ~~~~~~~~~~~~~~~~~

    Backends for Trashman.

    :Copyright: © 2011-2012, Kwpolska.
    :License: BSD (see /LICENSE).
"""

from .. import DS, _, TMError, __pyver__
import importlib


def select(backend):
    """Select the backend.  'auto' for auto-selection and 'info' for list."""
    backend = backend.strip().lower()
    # internal: [log/human, module, class, 'description as in docs', humans?]
    BACKENDS = {'auto': ['', '', '', _('[choose automatically]'), True],
                'config': ['', '', '', _('[consult trashman.cfg]'), True],
                'list': ['', '', '', _('[list backends]'), False],
                'dummy': ['dummy', 'dummytrash', 'DummyTrash', _('A dummy \
backend, printing all the requests it gets.'), False],
                'xdg': ['XDG', 'xdgtrash', 'XDGTrash', _('An XDG trash \
implementation.'), True]}
    if backend == 'config':
        backend = DS.config.get('default_backend', 'name')
        DS.log.info(_('*** backend from config: {}').format(backend))

    if backend == 'auto':
        # XDG is the only usable backend right now.
        backend = 'xdg'
    elif backend == 'list':
        col1 = len(max(BACKENDS.keys(), key=len))
        col2p = []
        col3p = []

        if __pyver__[0] == 3:
            for k, i in BACKENDS.items():
                col2p.append(i[0])
                col3p.append(i[3])
        else:
            for k, i in BACKENDS.iteritems():
                col2p.append(i[0])
                col3p.append(i[3])

        idword = '-b'
        nameword = _('NAME')
        dword = _('DESCRIPTION')
        col2 = len(max(col2p, key=len))
        col3 = len(max(col3p, key=len))

        if len(idword) > col1:
            col1 = len(idword)

        if len(nameword) > col2:
            col2 = len(nameword)

        space1 = (col1 - len(idword)) * ' '
        space2 = (col2 - len(nameword)) * ' '

        print(' '.join([idword + space1, '|', nameword + space2, '|', dword]))
        print('-+-'.join([col1 * '-', col2 * '-', col3 * '-']))

        if __pyver__[0] == 3:
            for k, i in BACKENDS.items():
                space1 = (col1 - len(k)) * ' '
                space2 = (col2 - len(i[0])) * ' '
                print(' '.join([k + space1, '|', i[0] + space2, '|', i[3]]))
        else:
            for k, i in BACKENDS.iteritems():
                space1 = (col1 - len(k)) * ' '
                space2 = (col2 - len(i[0])) * ' '
                print(' '.join([k + space1, '|', i[0] + space2, '|', i[3]]))

    if backend != 'list':
        if backend not in BACKENDS.keys():
            raise TMError(_('No such backend: {}').format(backend))
        else:
            b = BACKENDS[backend]
            mn = b[1]
            mc = b[2]
            m = importlib.import_module('.backends.{}'.format(mn), 'trashman')
            DS.trash = eval('m.{}'.format(mc))()
            DS.trashdir = DS.trash.trashdir
            DS.log.info(_('*** using the {} backend').format(b[0]))



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# Trashman v1.0.2
# A Python trash manager.
# Copyright (C) 2011-2012, Kwpolska.
# See /LICENSE for licensing information.

"""
    trashman.main()
    ~~~~~~~~~~~~~~~

    Trashman’s main routine.

    :Copyright: © 2011-2012, Kwpolska.
    :License: BSD (see /LICENSE).
"""

from . import DS, _, __version__
from .backends import select
import argparse


### main()         The main routine        ###
def main():
    """The main routine."""
    parser = argparse.ArgumentParser(description=_('Trashman -- a Python \
                                     trash manager.'))

    argopt = parser.add_argument_group(_('options'))
    argopr = parser.add_argument_group(_('operations'))

    parser.add_argument('-V', '--version', action='version',
                        version='Trashman v' + __version__)

    argopt.add_argument('-v', '--verbose', action='store_true', default=False,
                        dest='verbose', help=_('explain what is being done'))
    argopt.add_argument('-b', '--backend', action='store', default='config',
                        dest='backend', help=_('select the backend (default: \
                        config->auto->xdg)'))

    argopr.add_argument('-e', '--empty', action='store_true', default=False,
                        dest='empty', help=_('empty the trash and exit'))
    argopr.add_argument('-l', '--list', action='store_true', default=False,
                        dest='flist', help=_('list the files in trash and \
                                              exit'))
    argopr.add_argument('-r', '--restore', action='store_true', default=False,
                        dest='restore', help=_('restore FILE(s) from trash'))
    argopr.add_argument('-w', '--trash-location', action='store_true',
                        default=False, dest='showtloc', help=_('print the \
                        trash location and exit'))
    argopr.add_argument('-W', '--files-location', action='store_true',
                        default=False, dest='showfloc', help=_('print the \
                        trashed files location and exit'))
    parser.add_argument('files', metavar=_('FILE'), action='store', nargs='*',
                        help=_('files to trash'))
    args = parser.parse_args()

    exit_notrash = False

    select(args.backend)
    if args.backend == 'info':
        sys.exit(0)
    else:
        DS.trash.regenerate()

    if args.flist:
        DS.trash.list()
        exit_notrash = True

    if args.showtloc:
        print(DS.trash.trashdir)
        exit_notrash = True

    if args.showfloc:
        print(DS.trash.filedir)
        exit_notrash = True

    if args.restore:
        for fileres in args.files:
            DS.trash.restore(fileres, args.verbose)
        exit_notrash = True

    if args.empty:
        DS.trash.empty(args.verbose)
        exit_notrash = True

    if exit_notrash:
        exit()
    else:
        # Did not exit?  We can trash.
        for filedel in args.files:
            DS.trash.trash(filedel, args.verbose)



#!/usr/bin/python3
# -*- encoding: utf-8 -*-
# Trashman v1.0.2
# A Python trash manager.
# Copyright (C) 2011-2012, Kwpolska.
# See /LICENSE for licensing information.

"""
    trashman.TMDS
    ~~~~~~~~~~~~~
    Trashman Data Storage.

    :Copyright: © 2011-2012, Kwpolska.
    :License: BSD (see /LICENSE).
"""

from . import __version__
import sys
import os
import logging
try:
    import configparser
except ImportError:
    import ConfigParser as configparser


### TMDS           TM global data storage  ###
class TMDS(object):
    """Trashman Data Storage."""
    trash = None
    trashdir = None

    # Creating the configuration/log stuff...
    if sys.platform == 'darwin':
        confhome = os.path.expanduser('~/Library/Application Support/')
    else:
        confhome = os.getenv('XDG_CONFIG_HOME')
        if confhome is None:
            confhome = os.path.expanduser('~/.config/')

    kwdir = os.path.join(confhome, 'kwpolska')
    confdir = os.path.join(kwdir, 'trashman')

    if not os.path.exists(confhome):
        os.mkdir(confhome)

    if not os.path.exists(kwdir):
        os.mkdir(kwdir)

    if not os.path.exists(confdir):
        os.mkdir(confdir)

    if not os.path.exists(confdir):
        error(_('Cannot create the configuration directory.'))
        exit(1)

    config = configparser.SafeConfigParser()
    config.read(os.path.join(confdir, 'trashman.cfg'))
    if not config.has_section('default_backend'):
        config.add_section('default_backend')
        config.set('default_backend', 'name', 'auto')
        config.write(open(os.path.join(confdir, 'trashman.cfg'), 'w'))

    if not os.path.exists(confdir):
        try:
            os.mkdir(confdir)
        except:
            try:
                os.mkdir(confhome)
            except:
                pass

            try:
                os.mkdir(os.path.join(confhome, 'kwpolska'))
                os.mkdir(confdir)
            except:
                self.error('Cannot create the config directory ({}).'.format(
                    confdir))

    logging.basicConfig(format='%(asctime)-15s [%(levelname)-7s] \
:%(name)-10s: %(message)s', filename=confdir + '/trashman.log',
                        level=logging.DEBUG)
    log = logging.getLogger('trashman')

    log.info('*** Trashman v{}'.format(__version__))

    def info(self, msg):
        """Show an information and log it."""
        sys.stderr.write('* {}'.format(msg))
        self.log.info('(auto info    ) {}'.format(msg))

    def warning(self, msg):
        """Show a warning and log it."""
        sys.stderr.write('! {}'.format(msg))
        self.log.warning('(auto warning ) {}'.format(msg))

    def error(self, msg):
        """Show an error and log it."""
        sys.stderr.write('E {}'.format(msg))
        self.log.error('(auto error   ) {}'.format(msg))

    def critical(self, msg):
        """Show a critical error and log it."""
        sys.stderr.write('C {}'.format(msg))
        self.log.critical('(auto critical) {}'.format(msg))



#!/usr/bin/env python3

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(name='trashman',
      version='1.0.2',
      description='A Python trash manager.',
      author='Kwpolska',
      author_email='kwpolska@kwpolska.tk',
      url='https://github.com/Kwpolska/trashman',
      license='3-clause BSD',
      long_description=open('README.rst').read(),
      platforms='any',
      classifiers=['Development Status :: 5 - Production/Stable',
                   'Environment :: Console',
                   'Intended Audience :: End Users/Desktop',
                   'License :: OSI Approved :: BSD License',
                   'Natural Language :: English',
                   'Operating System :: POSIX :: Linux',
                   'Programming Language :: Python',
                   'Programming Language :: Python :: 2',
                   'Programming Language :: Python :: 3',
                   'Topic :: Utilities'],
      packages=['trashman', 'trashman.backends'],
      scripts=['bin/trash'],
      data_files=[('share/man/man8', ['docs/trashman.8.gz']),
                  ('share/locale/en/LC_MESSAGES', ['locale/en/LC_MESSAGES/\
trashman.mo']),
                  ('share/locale/pl/LC_MESSAGES', ['locale/pl/LC_MESSAGES/\
trashman.mo']),
                  ('share/zsh/site-functions', ['shell-completions/_trash'])])
